import json
from asset_model.oam_object import OAMObject
from abc import abstractmethod
from enum import Enum
from typing import List
from asset_model.asset import AssetType
from dataclasses import dataclass

class RelationType(str, Enum):
    BasicDNSRelation = "BasicDNSRelation"
    PortRelation = "PortRelation"
    PrefDNSRelation = "PrefDNSRelation"
    SimpleRelation = "SimpleRelation"
    SRVDNSRelation = "SRVDNSRelation"

class Relation(OAMObject):
    @property
    @abstractmethod
    def label(self) -> str:
        pass

    @property
    @abstractmethod
    def relation_type(self) -> RelationType:
        pass

RelationList = list(RelationType)

account_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "user": {RelationType.SimpleRelation: {AssetType.Person, AssetType.Organization}},
}

autnum_record_rels = {
    "whois_server": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "registrant": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "admin_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "abuse_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "technical_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "rdap_url": {RelationType.SimpleRelation: {AssetType.URL}},
}

autonomous_system_rels = {
    "announces": {RelationType.SimpleRelation: {AssetType.Netblock}},
    "registration": {RelationType.SimpleRelation: {AssetType.AutnumRecord}},
}

contact_record_rels = {
    "fqdn": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "person": {RelationType.SimpleRelation: {AssetType.Person}},
    "organization": {RelationType.SimpleRelation: {AssetType.Organization}},
    "location": {RelationType.SimpleRelation: {AssetType.Location}},
    "phone": {RelationType.SimpleRelation: {AssetType.Phone}},
    "url": {RelationType.SimpleRelation: {AssetType.URL}},
}

domain_record_rels = {
    "name_server": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "whois_server": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "registrar_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "registrant_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "admin_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "technical_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "billing_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
}

file_rels = {
    "url": {RelationType.SimpleRelation: {AssetType.URL}},
    "contains": {RelationType.SimpleRelation: {AssetType.ContactRecord, AssetType.URL}},
}

fqdn_rels = {
    "port": {RelationType.PortRelation: {AssetType.Service}},
    "dns_record": {
        RelationType.BasicDNSRelation: {AssetType.FQDN, AssetType.IPAddress, AssetType.Identifier},
        RelationType.PrefDNSRelation: {AssetType.FQDN},
        RelationType.SRVDNSRelation: {AssetType.FQDN},
    },
    "node": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "registration": {RelationType.SimpleRelation: {AssetType.DomainRecord}},
    "verified_for": {RelationType.SimpleRelation: {AssetType.Organization, AssetType.Service}},
}

funds_transfer_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "sender": {RelationType.SimpleRelation: {AssetType.Account}},
    "recipient": {RelationType.SimpleRelation: {AssetType.Account}},
    "third_party": {RelationType.SimpleRelation: {AssetType.Organization}},
}

identifier_rels = {
    "registration_agency": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "issuing_authority": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "issuing_agent": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
}

ip_rels = {
    "port": {RelationType.PortRelation: {AssetType.Service}},
    "ptr_record": {RelationType.SimpleRelation: {AssetType.FQDN}},
}

ipnet_record_rels = {
    "whois_server": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "registrant": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "admin_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "abuse_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "technical_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "rdap_url": {RelationType.SimpleRelation: {AssetType.URL}},
}

location_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
}

netblock_rels = {
    "contains": {RelationType.SimpleRelation: {AssetType.IPAddress}},
    "registration": {RelationType.SimpleRelation: {AssetType.IPNetRecord}},
}

org_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "legal_address": {RelationType.SimpleRelation: {AssetType.Location}},
    "hq_address": {RelationType.SimpleRelation: {AssetType.Location}},
    "location": {RelationType.SimpleRelation: {AssetType.Location}},
    "subsidiary": {RelationType.SimpleRelation: {AssetType.Organization}},
    "org_unit": {RelationType.SimpleRelation: {AssetType.Organization}},
    "account": {RelationType.SimpleRelation: {AssetType.Account}},
    "member": {RelationType.SimpleRelation: {AssetType.Person}},
    "website": {RelationType.SimpleRelation: {AssetType.URL}},
    "social_media_profile": {RelationType.SimpleRelation: {AssetType.URL}},
    "funding_source": {RelationType.SimpleRelation: {AssetType.Person, AssetType.Organization}},
}

person_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "address": {RelationType.SimpleRelation: {AssetType.Location}},
    "phone": {RelationType.SimpleRelation: {AssetType.Phone}},
    "account": {RelationType.SimpleRelation: {AssetType.Account}},
}

phone_rels = {
    "account": {RelationType.SimpleRelation: {AssetType.Account}},
    "contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
}

product_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "manufacturer": {RelationType.SimpleRelation: {AssetType.Organization}},
    "website": {RelationType.SimpleRelation: {AssetType.URL}},
    "release": {RelationType.SimpleRelation: {AssetType.ProductRelease}},
}

product_release_rels = {
    "id": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "website": {RelationType.SimpleRelation: {AssetType.URL}},
}

service_rels = {
    "provider": {RelationType.SimpleRelation: {AssetType.Organization}},
    "certificate": {RelationType.SimpleRelation: {AssetType.TLSCertificate}},
    "terms_of_service": {RelationType.SimpleRelation: {AssetType.File, AssetType.URL}},
    "product_used": {RelationType.SimpleRelation: {AssetType.Product, AssetType.ProductRelease}},
}

tlscert_rels = {
    "common_name": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "subject_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "issuer_contact": {RelationType.SimpleRelation: {AssetType.ContactRecord}},
    "san_dns_name": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "san_email_address": {RelationType.SimpleRelation: {AssetType.Identifier}},
    "san_ip_address": {RelationType.SimpleRelation: {AssetType.IPAddress}},
    "san_url": {RelationType.SimpleRelation: {AssetType.URL}},
    "issuing_certificate": {RelationType.SimpleRelation: {AssetType.TLSCertificate}},
    "issuing_certificate_url": {RelationType.SimpleRelation: {AssetType.URL}},
    "ocsp_server": {RelationType.SimpleRelation: {AssetType.URL}},
}

url_rels = {
    "domain": {RelationType.SimpleRelation: {AssetType.FQDN}},
    "ip_address": {RelationType.SimpleRelation: {AssetType.IPAddress}},
    "port": {RelationType.PortRelation: {AssetType.Service}},
    "file": {RelationType.SimpleRelation: {AssetType.File}},
}

def asset_type_relations(atype: AssetType) -> dict:
    match atype:
        case AssetType.Account:
            return account_rels
        case AssetType.AutnumRecord:
            return autnum_record_rels
        case AssetType.AutonomousSystem:
            return autonomous_system_rels
        case AssetType.ContactRecord:
            return contact_record_rels
        case AssetType.DomainRecord:
            return domain_record_rels
        case AssetType.File:
            return file_rels
        case AssetType.FQDN:
            return fqdn_rels
        case AssetType.FundsTransfer:
            return funds_transfer_rels
        case AssetType.Identifier:
            return identifier_rels
        case AssetType.IPAddress:
            return ip_rels
        case AssetType.IPNetRecord:
            return ipnet_record_rels
        case AssetType.Location:
            return location_rels
        case AssetType.Netblock:
            return netblock_rels
        case AssetType.Organization:
            return org_rels
        case AssetType.Person:
            return person_rels
        case AssetType.Phone:
            return phone_rels
        case AssetType.Product:
            return product_rels
        case AssetType.ProductRelease:
            return product_release_rels
        case AssetType.Service:
            return service_rels
        case AssetType.TLSCertificate:
            return tlscert_rels
        case AssetType.URL:
            return url_rels
        case _:
            return None

def get_asset_outgoing_relations(subject: AssetType) -> List[str] | None:
    """
    GetAssetOutgoingRelations returns the relation types allowed
    to be used when the subject is the asset type provided in the
    parameter.

    Providing an invalid subject causes a return value of nil.
    """
    relations = asset_type_relations(subject)
    if relations == None:
        return None

    return list(relations.keys())


def get_transform_asset_types(
        subject: AssetType,
        label: str,
        rtype: RelationType) -> List[AssetType] | None:
    """
    GetTransformAssetTypes returns the asset types allowed to be assigned
    when the subject is the asset type provided in the parameter, along
    with the provided label and RelationType.
    Providing an invalid subject causes a return value of nil."""
    relations = asset_type_relations(subject)
    if relations == None:
        return None

    label = label.lower()
    atypes: List[AssetType] = []
    for r, assets in relations.items():
        if r != label:
            continue
        
        for a in assets.values():
            atypes.extend(list(a))

    return atypes
        

def valid_relationship(
        src: AssetType,
        label: str,
        rtype: RelationType,
        dst: AssetType) -> bool:
    """
    ValidRelationship returns true if the relation is valid in the
    taxonomy when outgoing from the source asset type to the
    destination asset type.
    """
    atypes = get_transform_asset_types(src, label, rtype)
    if atypes == None:
        return False
    
    return dst in atypes



from dataclasses import dataclass
from dataclasses import field
from typing import Optional
from enum import Enum
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class Account(Asset):
    """Account represents an account managed by an organization.
    
    Should support relationships for the following:
    - User (e.g. Person or Organization)
    - Funds transfers
    - IBAN and SWIFT codes
    """
    id:             str = field(metadata={"json":"unique_id"})
    account_type:   str
    username:       Optional[str]   = None
    account_number: Optional[str]   = None
    balance:        Optional[float] = None
    active:         Optional[bool]  = None

    @property
    def key(self) -> str:
        return self.id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Account



from dataclasses import dataclass
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class AutonomousSystem(Asset):
    """AutonomousSystem represents an autonomous system."""
    number: int

    @property
    def key(self) -> str:
        return str(self.number)

    @property
    def asset_type(self) -> AssetType:
        return AssetType.AutonomousSystem



from dataclasses import dataclass
from dataclasses import field
from typing import Optional
from enum import Enum
from asset_model.property import PropertyType
from asset_model.asset import Asset, AssetType

@dataclass
class FundsTransfer(Asset):
    """FundsTransfer represents a single transfer of funds between two accounts.
    Should support relationships for the following:
    - Sender financial account (e.g. Account)
    - Recipient financial account (e.g. Account)
    - IBIN and SWIFT codes
    """
    id:               str = field(metadata={"json":"unique_id"})
    amount:           float
    reference_number: Optional[str] = None
    currency:         Optional[str] = None
    method:           Optional[str] = field(default=None, metadata={"json":"transfer_method"})
    exchange_date:    Optional[str] = None
    exchange_rate:    Optional[float] = None

    @property
    def key(self) -> str:
        return self.id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.FundsTransfer

    def to_dict(self) -> dict:
        return {key: value for key, value in {
            "unique_id":        self.id,
            "amount":           self.amount,
            "reference_number": self.reference_number,
            "currency":         self.currency,
            "transfer_method":  self.method,
            "exchange_date":    self.exchange_date,
            "exchange_rate":    self.exchange_rate,
        }.items() if value is not None}



from dataclasses import dataclass
from dataclasses import field
from typing import Optional
from enum import Enum
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class Product(Asset):
    """Product represents a technology product and organizes the
    product releases in the data model.
    
    Should support relationships for the following:
    - Manufacturer (e.g. Organization)
    - Website
    - Product releases
    """
    id:                str = field(metadata={"json":"unique_id"})
    name:              str = field(metadata={"json":"product_name"})
    type:              str = field(metadata={"json":"product_type"})
    category:          Optional[str] = None
    description:       Optional[str] = None
    country_of_origin: Optional[str] = None

    @property
    def key(self) -> str:
        return self.id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Product

@dataclass
class ProductRelease(Asset):
    """ProductRelease represents a release of a technology product
    that belongs to a Product.

    Should support relationships for the following:
    - Amazon Standard Identification Number (ASIN)
    - Global Trade Item Number (GTIN)
    - International Article Number (EAN)
    - International Standard Book Number (ISBN)
    - Manufacturer Part Number (MPN)
    - Model Number
    - NATO Stock Number (NSN)
    - Serial Number
    - Universal Product Code (UPC)
    - Version Number
    - Vulnerabilities
    - Website with release details

    """
    name: str
    release_date: Optional[str] = None

    @property
    def key(self) -> str:
        return self.name

    @property
    def asset_type(self) -> AssetType:
        return AssetType.ProductRelease



from dataclasses import dataclass
from dataclasses import field
from enum import Enum
from typing import Dict
import json
from asset_model.property import Property, PropertyType
from asset_model.relation import Relation, RelationType

@dataclass
class SimpleRelation(Relation):
    """SimpleRelation represents a simple relation in the graph with
    no additional data required."""
    name: str = field(metadata={"json":"label"})

    @property
    def label(self) -> str:
        return self.name

    @property
    def relation_type(self) -> RelationType:
        return RelationType.SimpleRelation

@dataclass
class SimpleProperty(Property):
    """SimpleProperty represents a simple property in the graph."""
    property_name: str
    property_value: str

    @property
    def name(self) -> str:
        return self.property_name

    @property
    def value(self) -> str:
        return self.property_value

    @property
    def property_type(self) -> PropertyType:
        return PropertyType.SimpleProperty

    def to_dict(self) -> Dict:
        return {
            'property_name': self.name,
            'property_value': self.value
        }



import pytest
import json
from asset_model.contact.location import Location
from asset_model.asset import Asset
from asset_model.asset import AssetType

def test_location_key():
    want = "123 Some Road New York NY"
    loc = Location(address=want)

    assert loc.key == want

def test_location_asset_type():
    assert issubclass(Location, Asset)

    loc = Location(address="example")
    assert loc.asset_type == AssetType.Location

def test_location_json():
    loc = Location(
        address="123 Main St",
        building="Building A",
        building_number="123",
        street_name="Main St",
        unit="Apt 1",
        po_box="P.O. Box 145",
        city="Anytown",
        locality="Anytown",
        province="Anyregion",
        country="US",
        postal_code="12345",
        gln=1234567890123,
    )

    expected_json = {
        "address": "123 Main St",
        "building": "Building A",
        "building_number": "123",
        "street_name": "Main St",
        "unit": "Apt 1",
        "po_box": "P.O. Box 145",
        "city": "Anytown",
        "locality": "Anytown",
        "province": "Anyregion",
        "country": "US",
        "postal_code": "12345",
        "gln": 1234567890123,
    }

    json_data = loc.to_dict()
    assert json_data == expected_json



from dataclasses import dataclass
from dataclasses import field
from typing import Optional, Dict, List
from asset_model.property import PropertyType
from asset_model.relation import RelationType
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class Service(Asset):
    """Service represents a service provided by an asset and/or organization.
    It should support relationships such as the following:
    - Provider (e.g. Organization)
    - Terms of service (e.g. File or URL)
    - TLS Certificate (e.g. TLSCertificate)
    - Product used to provide the service (e.g. Product or ProductRelease)
    """
    id:         str = field(metadata={"json":"unique_id"})
    type:       str = field(metadata={"json":"service_type"})
    output:     Optional[str] = None
    output_len: Optional[int] = field(default=None, metadata={"json":"output_length"})
    attributes: Optional[Dict[str, List[str]]] = None

    @property
    def key(self) -> str:
        return self.id

    @property
    def asset_type(self) -> AssetType:
        return AssetType.Service



import json
from asset_model.network.netblock import Netblock
from asset_model.network.netblock import NetblockType
from asset_model.asset import Asset
from asset_model.asset import AssetType

def test_netblock_key():
    want = "192.168.1.0/24"
    netblock = Netblock(cidr=want, type=NetblockType.IPv4) 
    assert netblock.key == want

def test_netblock_implements_asset():
    assert issubclass(Netblock, Asset)

def test_netblock(subtests):
    tests = [
        {
            "description": "Test successful creation of Netblock with valid IPv4 CIDR",
            "cidr": "198.51.100.0/24",
            "net_type": NetblockType.IPv4,
        },
        {
            "description": "Test successful creation of Netblock with valid IPv6 CIDR",
            "cidr": "2001:db8::/32",
            "net_type": NetblockType.IPv4,
        },
    ]
    
    for tt in tests:
        with subtests.test(tt["description"]):
            netblock = Netblock(cidr=tt["cidr"], type=tt["net_type"])

            assert netblock.cidr == tt["cidr"]
            assert netblock.type == tt["net_type"]
            assert netblock.asset_type == AssetType.Netblock

def test_netblock_json(subtests):
    tests = [
        {
            "description": "Test successful JSON serialization of Netblock with valid IPv4 CIDR",
            "cidr": "198.51.100.0/24",
            "type": NetblockType.IPv4,
        },
        {
            "description": "Test successful JSON serialization of Netblock with valid IPv6 CIDR",
            "cidr": "2001:db8::/32",
            "type": NetblockType.IPv6,
        },
    ]
    
    for tt in tests:
        with subtests.test(tt["description"]):
            netblock = Netblock(cidr=tt["cidr"], type=tt["type"])

            json_data = netblock.to_dict()
            
            expected_json = {"cidr": tt["cidr"], "type": tt["type"]}
            assert json_data == expected_json



from dataclasses import dataclass
from typing import List, Optional
from asset_model.asset import Asset
from asset_model.asset import AssetType


@dataclass
class AutnumRecord(Asset):
    """AutnumRecord represents the RDAP record for an autonomous
    system."""
    number:       int
    handle:       str
    name:         str
    created_date: str
    updated_date: str
    raw:          Optional[str] = None
    whois_server: Optional[str] = None
    status:       Optional[List[str]] = None

    @property
    def key(self) -> str:
        return self.handle

    @property
    def asset_type(self) -> AssetType:
        return AssetType.AutnumRecord



from dataclasses import dataclass
from asset_model.asset import AssetType
from asset_model.asset import Asset
from fqdn import FQDN as _FQDN

@dataclass
class FQDN(Asset):
    """FQDN represents a Fully Qualified Domain Name."""
    name: str
    
    @property
    def key(self) -> str:
        return self.name

    @property
    def asset_type(self) -> AssetType:
        return AssetType.FQDN

    @staticmethod
    def from_text(fqdn: str) -> 'FQDN':
        if not _FQDN(fqdn).is_valid:
            raise ValueError("invalid fqdn")
        return FQDN(name=fqdn)



import json
import pytest
from asset_model.pf.service import Service
from asset_model.property import PropertyType
from asset_model.relation import RelationType
from asset_model.asset import AssetType

def test_service_key():
    want = "222333444"
    serv = Service(id=want, type="HTTP")
    assert serv.key == want

def test_service_asset_type():
    s = Service(id="dummy", type="dummy")
    want = AssetType.Service
    assert s.asset_type == want

def test_service_json():
    s = Service(
        id="222333444",
        type="HTTP",
        output="Hello",
        output_len=5,
        attributes={"server": ["nginx-1.26.0"]}
    )

    expected_json = {
        "unique_id": "222333444",
        "service_type": "HTTP",
        "output": "Hello",
        "output_length": 5,
        "attributes": {"server": ["nginx-1.26.0"]}}
    json_data = s.to_dict()
    assert json_data == expected_json



from __future__ import annotations
from abc import ABC
from json import dumps
from dataclasses import field
from dataclasses import fields
from dataclasses import dataclass
from dataclasses import is_dataclass
from typing import Any
from enum import Enum

@dataclass(kw_only=True)
class OAMObject(ABC):
    extra: dict[str, Any] = field(default_factory=dict)
    
    def to_dict(self) -> dict:
        d = {}
        for field in fields(self):
            if field.name == "extra":
                continue
            json_name = field.metadata["json"] if "json" in field.metadata else field.name
            json_value = self.__dict__[field.name]
            if json_value is not None:
                if isinstance(json_value, Enum):
                    d[json_name] = json_value.value
                else:  
                    d[json_name] = json_value

        extra_d = { f"extra_{k}": v for k, v in self.extra.items() }

        d.update(extra_d)
            
        return d
    
    def to_json(self) -> str:
        return dumps(self.to_dict())        

    def equals(self, to: OAMObject):
        return self.to_dict() == to.to_dict()



import pytest
import json
from asset_model.pf.vuln import VulnProperty
from asset_model.property import Property
from asset_model.property import PropertyType

def test_vuln_property_name():
    want = "CVE-2019-00001"
    sp = VulnProperty(
        id="CVE-2019-00001",
        description="In macOS before 2.12.6, there is a vulnerability in the RPC...",
        source="Tenable",
        category="Firewall",
        enumeration="CVE",
        reference="https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2019-6111",
    )
    assert sp.name == want

def test_vuln_property_value():
    want = "In macOS before 2.12.6, there is a vulnerability in the RPC..."
    sp = VulnProperty(
        id="CVE-2019-00001",
        description="In macOS before 2.12.6, there is a vulnerability in the RPC...",
        source="Tenable",
        category="Firewall",
        enumeration="CVE",
        reference="https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2019-6111",
    )
    assert sp.value == want

def test_vuln_property_implements_property():
    assert issubclass(VulnProperty, Property)

def test_vuln_property_creation():
    sp = VulnProperty(
        id="CVE-2019-00001",
        description="In macOS before 2.12.6, there is a vulnerability in the RPC...",
        source="Tenable",
        category="Firewall",
        enumeration="CVE",
        reference="https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2019-6111",
    )

    assert sp.id == "CVE-2019-00001"
    assert sp.description == "In macOS before 2.12.6, there is a vulnerability in the RPC..."
    assert sp.source == "Tenable"
    assert sp.category == "Firewall"
    assert sp.enumeration == "CVE"
    assert sp.reference == "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2019-6111"
    assert sp.property_type == PropertyType.VulnProperty

def test_vuln_property_json_serialization():
    sp = VulnProperty(
        id="CVE-2019-00001",
        description="foobar",
        source="Tenable",
        category="Firewall",
        enumeration="CVE",
        reference="URL",
    )

    json_data = sp.to_dict()
    expected_json = { 
        "id": "CVE-2019-00001",
        "desc": "foobar",
        "source": "Tenable",
        "category": "Firewall",
        "enum": "CVE",
        "ref": "URL"
    }
    assert json_data == expected_json



from dataclasses import dataclass
from typing import List
from asset_model.asset import Asset
from asset_model.asset import AssetType
from enum import Enum

class TLSKeyUsageType(str, Enum):
    DigitalSignature = "Digital Signature"
    ContentCommitment = "Content Commitment"
    KeyEncipherment = "Key Encipherment"
    DataEncipherment = "Data Encipherment"
    KeyAgreement = "Key Agreement"
    CertSign = "Certificate Sign"
    CRLSign = "CRL Sign"
    EncipherOnly = "Encipher Only"
    DecipherOnly = "Decipher Only"

class TLSExtKeyUsageType(str, Enum):
    Any = "Any Usage"
    ServerAuth = "TLS Server Authentication"
    ClientAuth = "TLS Client Authentication"
    CodeSigning = "Code Signing"
    EmailProtection = "E-mail Protection"
    IPSECEndSystem = "IPSec End System"
    IPSECTunnel = "IPSec Tunnel"
    IPSECUser = "IPSec User"
    TimeStamping = "Trusted Timestamping"
    OCSPSigning = "OCSP Signing"
    MicrosoftServerGatedCrypto = "Microsoft Server Gated Crypto"
    NetscapeServerGatedCrypto = "Netscape Server Gated Crypto"
    MicrosoftCommercialCodeSigning = "Microsoft Commercial Code Signing"
    MicrosoftKernelCodeSigning = "Microsoft Kernel Code Signing"
    
@dataclass
class TLSCertificate(Asset):
    """TLSCertificate represents a TLS Certificate asset."""
    version:                  str
    serial_number:            str
    subject_common_name:      str
    issuer_common_name:       str
    not_before:               str
    not_after:                str
    key_usage:                List[TLSKeyUsageType]
    ext_key_usage:            List[TLSExtKeyUsageType]
    signature_algorithm:      str
    public_key_algorithm:     str
    is_ca:                    bool
    crl_distribution_points:  List[str]
    subject_key_id:           str
    authority_key_id:         str

    @property
    def key(self) -> str:
        return self.serial_number

    @property
    def asset_type(self) -> AssetType:
        return AssetType.TLSCertificate



import json
from asset_model.oam_object import OAMObject
from abc import abstractmethod
from enum import Enum
from dataclasses import dataclass

class PropertyType(str, Enum):
    DNSRecordProperty = "DNSRecordProperty"
    SimpleProperty = "SimpleProperty"
    SourceProperty = "SourceProperty"
    VulnProperty = "VulnProperty"

class Property(OAMObject):
    @property
    @abstractmethod
    def name(self) -> str:
        pass

    @property
    @abstractmethod
    def value(self) -> str:
        pass

    @property
    @abstractmethod
    def property_type(self) -> PropertyType:
        pass

PropertyList = list(PropertyType)



from enum import Enum
from typing import Optional
from dataclasses import dataclass
from dataclasses import field
from asset_model.relation import Relation
from asset_model.relation import RelationType
from asset_model.property import Property
from asset_model.property import PropertyType
from asset_model.oam_object import OAMObject

@dataclass
class BasicDNSRelation(Relation):
    """BasicDNSRelation is a relation in the graph representing a
    basic DNS resource record."""
    name:    str = field(metadata={"json":"label"})
    rrtype:  int = field(metadata={"json":"header_rrtype"})
    rrname:  Optional[str] = field(metadata={"json":"header_rrname"})
    cls:     Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:     Optional[int] = field(default=None, metadata={"json":"header_ttl"})


    @property
    def relation_type(self) -> RelationType:
        return RelationType.BasicDNSRelation

    @property
    def label(self) -> str:
        return self.name

@dataclass
class PrefDNSRelation(Relation):
    """PrefDNSRelation is a relation in the graph representing a DNS
    resource record with preference information."""
    name:       str = field(metadata={"json":"label"})
    preference: int
    rrtype:     int = field(metadata={"json":"header_rrtype"})
    rrname:     Optional[str] = field(metadata={"json":"header_rrname"})
    cls:        Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:        Optional[int] = field(default=None, metadata={"json":"header_ttl"})

    @property
    def relation_type(self) -> RelationType:
        return RelationType.PrefDNSRelation

    @property
    def label(self) -> str:
        return self.name

@dataclass
class SRVDNSRelation(Relation):
    """SRVDNSRelation is a relation in the graph representing a DNS
    SRV resource record."""
    name:     str = field(metadata={"json":"label"})
    priority: int
    weight:   int
    port:     int
    rrtype:   int = field(metadata={"json":"header_rrtype"})
    rrname:   Optional[str] = field(metadata={"json":"header_rrname"})
    cls:      Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:      Optional[int] = field(default=None, metadata={"json":"header_ttl"})

    @property
    def relation_type(self) -> RelationType:
        return RelationType.SRVDNSRelation
    
    @property
    def label(self) -> str:
        return self.name

@dataclass
class DNSRecordProperty(Property):
    """DNSRecordProperty represents a DNS resource record that does
    not refer to another asset in the graph."""
    property_name: str
    data:          str
    rrtype:        int = field(metadata={"json":"header_rrtype"})
    rrname:        Optional[str] = field(metadata={"json":"header_rrname"})
    cls:           Optional[int] = field(default=None, metadata={"json":"header_class"})
    ttl:           Optional[int] = field(default=None, metadata={"json":"header_ttl"})

    @property
    def property_type(self) -> PropertyType:
        return PropertyType.DNSRecordProperty
    
    @property
    def name(self) -> str:
        return self.property_name

    @property
    def value(self) -> str:
        return self.data



from dataclasses import dataclass
from dataclasses import field
from typing import Optional, List
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class IPNetRecord(Asset):
    """IPNetRecord represents the RDAP record for an IP network."""
    cidr:          str
    handle:        str
    type:          str
    name:          str
    created_date:  str
    updated_date:  str
    start_address: str
    end_address:   str
    raw:           Optional[str] = None
    method:        Optional[str] = None
    country:       Optional[str] = None
    parent_handle: Optional[str] = None
    whois_server:  Optional[str] =  None
    status:        Optional[List[str]] =  None

    @property
    def key(self) -> str:
        return self.handle

    @property
    def asset_type(self) -> AssetType:
        return AssetType.IPNetRecord



from dataclasses import dataclass
from dataclasses import field
from typing import Optional
from urllib.parse import urlparse
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class URL(Asset):
    """URL represents a URL."""
    raw:      str           = field(metadata={"json":"url"})
    scheme:   Optional[str] = None
    host:     Optional[str] = None
    path:     Optional[str] = None
    username: Optional[str] = None
    password: Optional[str] = None
    port:     Optional[int] = None
    options:  Optional[str] = None
    fragment: Optional[str] = None

    @property
    def key(self) -> str:
        return self.raw

    @property
    def asset_type(self) -> AssetType:
        return AssetType.URL

    @staticmethod
    def from_text(url: str) -> 'URL':
        o = urlparse(url)
        return URL(
            raw=url,
            scheme=o.scheme,
            host=o.hostname or "",
            path=o.path,
            username=o.username,
            password=o.password,
            port=o.port,
            options=o.query,
            fragment=o.fragment
        )



from dataclasses import dataclass
from typing import Optional, List
import json
import pytest
from asset_model.registration.ipnet_record import IPNetRecord
from asset_model.asset import Asset, AssetType

def test_ipnet_record_key():
    want = "NET-150-154-0-0-1"
    as_record = IPNetRecord(handle=want, cidr="150.154.0.0/16",
                             start_address="150.154.0.0",
                             end_address="150.154.255.255",
                             type="IPv4", name="REV-MVCC", created_date="1991-05-20",
                             updated_date="2024-03-28")
    
    assert as_record.key == want


def test_ipnet_record_asset_type():
    assert issubclass(IPNetRecord, Asset)
    assert isinstance(IPNetRecord, type)

    w = IPNetRecord(cidr="150.154.0.0/16", handle="NET-150-154-0-0-1", 
                    start_address="150.154.0.0", 
                    end_address="150.154.255.255",
                    type="IPv4", name="REV-MVCC", created_date="1991-05-20", 
                    updated_date="2024-03-28")
    
    want = AssetType.IPNetRecord

    assert w.asset_type == want


def test_ipnet_record():
    record = IPNetRecord(
        cidr="150.154.0.0/16",
        handle="NET-150-154-0-0-1",
        start_address="150.154.0.0",
        end_address="150.154.255.255",
        type="IPv4",
        name="REV-MVCC",
        method="DIRECT ALLOCATION",
        parent_handle="NET-150-0-0-0-0",
        whois_server="whois.arin.net",
        created_date="1991-05-20 04:00:00",
        updated_date="2024-03-28 18:47:50",
        status=["active"],
    )

    # Test AssetType method
    assert record.asset_type == AssetType.IPNetRecord

    # Test JSON method
    expected_json = {
        "cidr": "150.154.0.0/16",
        "handle": "NET-150-154-0-0-1",
        "start_address": "150.154.0.0",
        "end_address": "150.154.255.255",
        "type": "IPv4",
        "name": "REV-MVCC",
        "method": "DIRECT ALLOCATION",
        "parent_handle": "NET-150-0-0-0-0",
        "whois_server": "whois.arin.net",
        "created_date": "1991-05-20 04:00:00",
        "updated_date": "2024-03-28 18:47:50",
        "status": ["active"]}
    
    json_data = record.to_dict()
    assert json_data == expected_json



from __future__ import annotations
from dataclasses import fields
from dataclasses import is_dataclass
import inspect
import re
from typing import cast
from typing import Type, TypeVar, Mapping, Any
from asset_model.oam_object import OAMObject
import asset_model
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from _typeshed import DataclassInstance

def _get_oam_obj_by_name(name: str) -> Type[OAMObject]:
    for [_name, cls] in inspect.getmembers(asset_model, inspect.isclass):
        if _name == name:
            return cast(type[OAMObject], cls)

    raise Exception("unsupported oam object")

def get_property_by_type(type: asset_model.PropertyType) -> Type[asset_model.Property]:
    if type not in asset_model.PropertyList:
        raise Exception("unsupported relation type")
    return cast(Type[asset_model.Property], _get_oam_obj_by_name(type.value))

def get_relation_by_type(type: asset_model.RelationType) -> Type[asset_model.Relation]:
    if type not in asset_model.RelationList:
        raise Exception("unsupported relation type")
    return cast(Type[asset_model.Relation], _get_oam_obj_by_name(type.value))

def get_asset_by_type(type: asset_model.AssetType) -> Type[asset_model.Asset]:
    if type not in asset_model.AssetList:
        raise Exception("unsupported asset type")
    return cast(Type[asset_model.Asset], _get_oam_obj_by_name(type.value))

def describe_oam_object(o: Type[OAMObject]) -> list:
    d = []
    for field in fields(o):
        json_name = field.metadata["json"] if "json" in field.metadata else field.name
        d.append(json_name)
            
    return d

T = TypeVar("T", bound="OAMObject")

def make_oam_object_from_dict(o: Type[T], d: Mapping[str, Any]) -> T:
    real_d = {}
    o_fields = fields(cast(Any, o))
    for key, value in d.items():
        for field in o_fields:
            if ("json" in field.metadata and field.metadata["json"] == key) \
               or field.name == key:
                real_d[field.name] = value
                break

    instance = o(**real_d)

    extra_keys = list(filter(lambda e: e.startswith("extra_"), d.keys()))
    for key in extra_keys:
        real_key = re.sub(r"^extra_", "", key)
        instance.extra[real_key] = d[key]
        
    return instance



import pytest
import json
from asset_model.account.account import Account
from asset_model.asset import Asset
from asset_model.asset import AssetType

def test_account_key():
    expected = "222333444"
    a = Account(
        id=expected,
        username="test",
        account_number="12345",
        account_type="ACH"
    )

    assert a.key == expected

def test_account_asset_type():
    assert issubclass(Account, Asset)

    a = Account(id="test_id", account_type="test")
    expected = AssetType.Account
    actual = a.asset_type

    assert actual == expected

def test_account_json():
    a = Account(
        id="222333444",
        account_type="ACH",
        username="test",
        account_number="12345",
        balance=42.50,
        active=True
    )
    expected = {
        "unique_id": "222333444",
        "account_type": "ACH",
        "username": "test",
        "account_number": "12345",
        "balance": 42.50,
        "active": True
    }
    actual = a.to_dict()

    assert actual == expected



import pytest
import json
from asset_model.general.simple import SimpleRelation
from asset_model.general.simple import SimpleProperty
from asset_model.property import Property
from asset_model.property import PropertyType
from asset_model.relation import Relation
from asset_model.relation import RelationType


def test_simple_relation_name():
    want = "anything"
    sr = SimpleRelation(name=want)
    
    assert sr.label == want


def test_simple_relation_implements_relation():
    assert issubclass(SimpleRelation, Relation)


def test_simple_relation():
    sr = SimpleRelation(name="anything")

    assert sr.name == "anything"
    assert sr.relation_type == RelationType.SimpleRelation

    expected = {"label":"anything"}
    assert sr.to_dict() == expected


def test_simple_property_name():
    want = "anything"
    sp = SimpleProperty(property_name="anything", property_value="foobar")

    assert sp.name == want


def test_simple_property_value():
    want = "foobar"
    sp = SimpleProperty(property_name="anything", property_value="foobar")

    assert sp.value == want


def test_simple_property_implements_property():
    assert issubclass(SimpleProperty, Property)


def test_simple_property():
    sp = SimpleProperty(property_name="anything", property_value="foobar")

    assert sp.property_name == "anything"
    assert sp.property_value == "foobar"
    assert sp.property_type == PropertyType.SimpleProperty

    expected_json = {
        "property_name":"anything",
        "property_value":"foobar"
    }
    assert sp.to_dict() == expected_json



from .port import PortRelation
from .simple import SimpleProperty
from .simple import SimpleRelation
from .source import SourceProperty
from .identifier import Identifier
from .identifier import IdentifierType



import json
import pytest
from asset_model.general.identifier import Identifier
from asset_model.general.identifier import IdentifierType
from asset_model.asset import AssetType

def test_identifier_key():
    want = "Legal Entity Identifier:549300XMYB546ZI1F126"
    i = Identifier(
        unique_id=want,
        id="549300XMYB546ZI1F126",
        type=IdentifierType.LEICode
    )
    
    assert i.key == want

def test_identifier_asset_type():
    i = Identifier(unique_id="", id="",
        type=IdentifierType.LEICode)
    
    expected = AssetType.Identifier
    actual = i.asset_type
    
    assert actual == expected

def test_identifier_json():
    i = Identifier(
        unique_id="Legal Entity Identifier:549300XMYB546ZI1F126",
        id="549300XMYB546ZI1F126",
        type=IdentifierType.LEICode,
        creation_date="2013-07-24T14:15:00Z",
        update_date="2023-08-04T17:33:45Z",
        expiration_date="2020-01-16T00:32:00Z",
        status="ACTIVE"
    )
    
    expected = {
        "unique_id":"Legal Entity Identifier:549300XMYB546ZI1F126",
        "id":"549300XMYB546ZI1F126",
        "id_type":"lei",
        "creation_date":"2013-07-24T14:15:00Z",
        "update_date":"2023-08-04T17:33:45Z",
        "expiration_date":"2020-01-16T00:32:00Z",
        "status":"ACTIVE"
    }

    assert i.to_dict() == expected



from dataclasses import dataclass
from asset_model.asset import Asset
from asset_model.asset import  AssetType
from enum import Enum
from ipaddress import ip_address
from ipaddress import IPv4Address, IPv6Address

class IPAddressType(str, Enum):
    IPv4 = "IPv4"
    IPv6 = "IPv6"

@dataclass
class IPAddress(Asset):
    """IPAddress represents an IP address."""
    address: str
    type:    IPAddressType

    @property
    def key(self) -> str:
        return self.address

    @property
    def asset_type(self) -> AssetType:
        return AssetType.IPAddress

    @staticmethod
    def from_text(address: str) -> 'IPAddress':
        try:
            ip_type = IPAddressType.IPv4 \
                if type(ip_address(address)) is IPv4Address \
                else IPAddressType.IPv6
        except ValueError as e:
            raise e
            
        return IPAddress(address, ip_type)



import pytest
from asset_model.dns.dns_record import BasicDNSRelation
from asset_model.dns.dns_record import PrefDNSRelation
from asset_model.dns.dns_record import SRVDNSRelation
from asset_model.dns.dns_record import DNSRecordProperty
from asset_model.relation import Relation
from asset_model.relation import RelationType
from asset_model.property import Property
from asset_model.property import PropertyType
import json

def test_basic_dns_relation_name():
    want = "dns_record"
    br = BasicDNSRelation(
        name=want,
        rrtype=1,
        rrname="A",
        cls=1,
        ttl=86400
    )

    assert br.label == want

def test_basic_dns_relation_implements_relation():
    assert issubclass(BasicDNSRelation, Relation)

def test_basic_dns_relation():
    dr = BasicDNSRelation(
        name="dns_record",
        rrtype=1, rrname="A", cls=1, ttl=86400,
    )

    assert dr.name == "dns_record"
    assert dr.rrtype == 1
    assert dr.rrname == "A"
    assert dr.cls == 1
    assert dr.ttl == 86400
    assert dr.relation_type == RelationType.BasicDNSRelation

    json_data = dr.to_dict()
    expected_json = {
        "label": "dns_record",
        "header_rrtype": 1,
        "header_rrname": "A",
        "header_class": 1,
        "header_ttl": 86400
    }
    assert json_data == expected_json

def test_pref_dns_relation_name():
    want = "dns_record"
    br = PrefDNSRelation(
        name=want,
        rrtype=1, rrname="A", cls=1, ttl=86400,
        preference=5,
    )

    assert br.label == want

def test_pref_dns_relation_implements_relation():
    assert issubclass(PrefDNSRelation, Relation)

def test_pref_dns_relation():
    pr = PrefDNSRelation(
        name="dns_record",
        rrtype=1, rrname="A", cls=1, ttl=86400,
        preference=5,
    )

    assert pr.name == "dns_record"
    assert pr.rrtype == 1
    assert pr.rrname == "A"
    assert pr.cls == 1
    assert pr.ttl == 86400
    assert pr.preference == 5
    assert pr.relation_type is RelationType.PrefDNSRelation

    json_data = pr.to_dict()
    expected_json = {
        "label": "dns_record",
        "preference": 5,
        "header_rrtype": 1,
        "header_rrname": "A",
        "header_class": 1,
        "header_ttl": 86400,
    }
    assert json_data == expected_json

def test_srv_dns_relation_name():
    want = "dns_record"
    br = SRVDNSRelation(
        name="dns_record",
        rrtype=1, rrname="A", cls=1, ttl=86400,
        priority=10,
        weight=5,
        port=80
    )

    assert br.label == want

def test_srv_dns_relation_implements_relation():
    assert issubclass(SRVDNSRelation, Relation)

def test_srv_dns_relation():
    sr = SRVDNSRelation(
        name="dns_record",
        rrtype=1, rrname="A", cls=1, ttl=86400,
        priority=10,
        weight=5,
        port=80
    )

    assert sr.name == "dns_record"
    assert sr.rrtype == 1
    assert sr.rrname == "A"
    assert sr.cls == 1
    assert sr.ttl == 86400
    assert sr.priority == 10
    assert sr.weight == 5
    assert sr.port == 80
    assert sr.relation_type is RelationType.SRVDNSRelation

    json_data = sr.to_dict()
    expected_json = {
        "label": "dns_record",
        "header_rrtype": 1,
        "header_rrname": "A",
        "header_class": 1,
        "header_ttl": 86400,
        "priority": 10,
        "weight": 5,
        "port": 80,
    }
    assert json_data == expected_json

def test_dns_record_property_name():
    p = DNSRecordProperty(
        property_name="anything",
        rrtype=1, rrname="A", cls=1, ttl=86400,
        data="foobar"
    )

    assert p.name == "anything"

def test_dns_record_property_value():
    p = DNSRecordProperty(
        property_name="anything",
        rrtype=1, rrname="A", cls=1, ttl=86400,
        data="foobar"
    )

    assert p.value == "foobar"

def test_dns_record_property_implements_property():
    assert issubclass(DNSRecordProperty, Property)

def test_dns_record_property():
    p = DNSRecordProperty(
        property_name="anything",
        rrtype=16, rrname="TXT", cls=1, ttl=86400,
        data="foobar"
    )

    assert p.property_name == "anything"
    assert p.data == "foobar"
    assert p.property_type is PropertyType.DNSRecordProperty

    json_data = p.to_dict()
    expected_json = {
        "property_name": "anything",
        "data": "foobar",
        "header_rrtype": 16,
        "header_rrname": "TXT",
        "header_class": 1,
        "header_ttl": 86400,
    }
    assert json_data == expected_json



from dataclasses import dataclass
from dataclasses import field
from asset_model.relation import Relation, RelationType
import json

@dataclass
class PortRelation(Relation):
    """PortRelation is a relation in the graph representing an open
    port."""
    name: str = field(metadata={"json":"label"})
    port_number: int
    protocol: str
    
    @property
    def label(self) -> str:
        return self.name

    @property
    def relation_type(self) -> RelationType:
        return RelationType.PortRelation



import json
from asset_model.url.url import URL
from asset_model.asset import Asset
from asset_model.asset import AssetType

def test_url_key():
    want = "https://owasp.org/"
    url = URL(want)
    assert url.key == want
        
def test_url_asset_type():
    url = URL("https://owasp.org/")
    assert url.asset_type == AssetType.URL

def test_url_implements_asset():
    assert issubclass(URL, Asset)
    
def test_url_json():
    url = URL(
	raw="http://user:pass@example.com:8080/path?option1=value1&option2=value2#fragment",
        scheme="http",
        username="user",
	password="pass",
	host="example.com",
	port=8080,
	path="/path",
	options="option1=value1&option2=value2",
	fragment="fragment",
    )

    expectedJSON = {
        "url":"http://user:pass@example.com:8080/path?option1=value1&option2=value2#fragment",
        "scheme":"http",
        "host":"example.com",
        "path":"/path",
        "username":"user",
        "password":"pass",
        "port":8080,
        "options":"option1=value1&option2=value2",
        "fragment":"fragment"
    }
    
    assert json.dumps(expectedJSON) == url.to_json()
    
def test_url_from_text():
    url = URL.from_text("http://user:pass@example.com:8080/path?option1=value1&option2=value2#fragment")

    want = URL(
        raw="http://user:pass@example.com:8080/path?option1=value1&option2=value2#fragment",
        scheme="http",
        host="example.com",
        path="/path",
        username="user",
        password="pass",
        port=8080,
        options="option1=value1&option2=value2",
        fragment="fragment"
    )
    
    assert url.raw == want.raw
    assert url.scheme == want.scheme
    assert url.host == want.host
    assert url.path == want.path
    assert url.username == want.username
    assert url.password == want.password
    assert url.port == want.port
    assert url.options == want.options
    assert url.fragment == want.fragment



from dataclasses import dataclass
from asset_model.asset import Asset
from asset_model.asset import AssetType

@dataclass
class ContactRecord(Asset):
    """ContactRecord links various assets together to form complete contact information."""
    discovered_at: str

    @property
    def key(self) -> str:
        return self.discovered_at

    @property
    def asset_type(self) -> AssetType:
        return AssetType.ContactRecord



from asset_model import AssetType, RelationType
from asset_model import get_asset_by_type, get_relation_by_type
from asset_model import describe_oam_object

fqdn_cls = get_relation_by_type(RelationType.BasicDNSRelation)





import json
import pytest

from asset_model.network.ip_address import IPAddress
from asset_model.network.ip_address import IPAddressType
from asset_model.asset import Asset
from asset_model.asset import AssetType

def test_ip_address_key():
    want = "1.1.1.1"
    ip = IPAddress(address=want, type=IPAddressType.IPv4) 
    assert ip.key == want

def test_ip_address_asset():
    assert issubclass(IPAddress, Asset)

def test_ip_address(subtests):
    tests = [
        {
            "description": "Test successful creation of IPAddress with valid IPv4 address",
            "ip": "192.168.1.1",
            "net_type": IPAddressType.IPv4,
        },
        {
            "description": "Test successful creation of IPAddress with valid IPv6 address",
            "ip": "2001:db8::1",
            "net_type": IPAddressType.IPv6,
        },
    ]
    
    for tt in tests:
        with subtests.test(tt["description"]):
            ip = tt["ip"]
            ip_address = IPAddress(address=ip, type=tt["net_type"])

            assert ip_address.address is not None
            assert ip_address.address == ip
            assert ip_address.type is tt["net_type"]
            assert ip_address.asset_type is AssetType.IPAddress

def test_ip_address_json(subtests):
    tests = [
        {
            "description": "Test successful JSON serialization of IPAddress with valid IPv4 address",
            "ip": "192.168.1.1",
            "net_type": IPAddressType.IPv4,
        },
        {
            "description": "Test successful JSON serialization of IPAddress with valid IPv6 address",
            "ip": "2001:db8::1",
            "net_type": IPAddressType.IPv6,
        },
    ]
    
    for tt in tests:
        with subtests.test(tt["description"]):
            ip = tt["ip"]
            ip_address = IPAddress(address=ip, type=tt["net_type"])

            json_data = ip_address.to_dict()
            expected_json = {"address": tt["ip"], "type": tt["net_type"]}
            assert json_data == expected_json



import json
import pytest
from asset_model.registration.domain_record import DomainRecord
from asset_model.asset import AssetType
from asset_model.asset import Asset

def test_domain_record_key():
    want = "owasp.org"
    dr = DomainRecord(domain=want, raw="", id="")
    assert dr.key == want

def test_domain_record_asset_type():
    assert issubclass(DomainRecord, Asset)  # Verify proper implementation of the Asset interface
    w = DomainRecord(raw="", id="", domain="")
    want = AssetType.DomainRecord
    assert w.asset_type == want

def test_domain_record():
    record = DomainRecord(
        domain="example.com",
        created_date="2020-01-01",
        updated_date="2021-01-01",
        expiration_date="2022-01-01",
        status=["active", "clientTransferProhibited"],
        dnssec=True,
    )

    # Test AssetType method
    assert record.asset_type == AssetType.DomainRecord

    # Test JSON method
    expected_json = {
        "domain":"example.com",
        "created_date":"2020-01-01",
        "updated_date":"2021-01-01",
        "expiration_date":"2022-01-01",
        "status":["active","clientTransferProhibited"],
        "dnssec":True
    }
    actual_json = record.to_dict()
    assert actual_json == expected_json
